//
//  TIAPurchase.h
//  TOAST In App Purchase
//
//  Created by NHNENT on 2014. 10. 22..
//  Copyright (c) 2014년 NHN Entertainment. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>

/** @constant TOAST In App Purchase SDK Version */
#define TIA_PURCHASE_SDK_VERSION_STRING @"1.4.4"

/** @constant TOAST In App Purchase Error Domain  */
extern NSString *const TIAPurchaseErrorDomain;

/*!
 @typedef TIAPurchaseErrorCode enum
 
 @abstract Describes what error code (if any)
 */
typedef NS_ENUM(NSUInteger, TIAPurchaseErrorCode) {
    /** Network timeout error */
    TIAErrorNetworkTimeout              = 100,
    /** Should be register user identification */
    TIAErrorAuthorization               = 101,
    /** Unsupported device */
    TIAErrorUnsupportedDevice           = 102,
    /** Unsupported market */
    TIAErrorUnsupportedStore            = 103,
    /** User canceled */
    TIAErrorUserCanceled                = 104,
    /** Initialized error */
    TIAErrorInitializedError            = 105,
    /** HTTP response code is not 20x */
    TIAErrorServerUnknownError          = 106,
    /** API response is failure */
    TIAErrorApiResponseError            = 107,
    /** App Store occured error */
    TIAErrorAppStoreError               = 108,
    /** App Store occured purchase process error */
    TIAErrorAppStorePurchaseError       = 109,
    /** Cash was insufficient for ONGATE */
    TIAErrorOnGateCashError             = 115
};

/*!
 @typedef TIARequestHandler
 
 @abstract
 A block that to register for a callback with the results of that request once the connection completes.
 
 @discussion
 The call occurs on the UI Thread.
 
 @param result  The result of the request. This is a translation of JSON data to 'NSDictionany' and 'NSArray' objects.
 This is nil if there was an error.
 
 @param error   The 'NSError' representing any error that occurred.
 */

typedef void(^TIARequestHandler)(id result, NSError *error);


/*!
 @class TIAPurchase
 
 @abstract
 The 'TIAPurchase' represents a single connection to In App Purchase to service a request.
 */

@interface TIAPurchase : NSObject


/*!
 @method
 
 @abstract
 Enable debug mode
 
 @param isDebuggable if isDebuggable value is true, Enable to debug mode
 */

+ (void) setDebugMode:(BOOL) isDebuggable;

/*!
 @method
 
 @abstract
 Simple method to regist appId
 
 @param appId   represent appId which is registered in Web Console
 
 @param error   The 'NSError' representing any error that occured
 
 */
+ (BOOL) registerAppId:(NSString *)appId error:(NSError **)error;


/*!
 @method
 
 @abstract
 Simple method to regist store id
 
 @param storeId   represent storeId. you can register 'AS' or 'ONGATE'
 
 @param error   The 'NSError' representing any error that occured
 
 */
+ (BOOL) registerStoreId:(NSString *)storeId error:(NSError **)error;


/*!
 @method
 
 @abstract
 Simple method to server phase
 
 @param serverPhase   represent server phase. you can register 'ALPHA' or 'BETA' or 'REAL' (default. REAL)
 
 @param error   The 'NSError' representing any error that occured
 
 */
+ (BOOL) registerServerPhase:(NSString *)serverPhase error:(NSError **)error;



/*!
 @method
 
 @abstract
 Simple method to register user identification.
 
 @param userId  represent to user identification
 
 @param error   The 'NSError' representing any error that occurred.
 */

+ (BOOL) registerUserId:(NSString *)userId
                  error:(NSError **)error;



/*!
 @method
 
 @abstract
 Starts a request for purchase process
 
 @param baseViewController  represent to current view controller
 
 @param itemId              represent to item id
 
 @param handler             handler's result has that translation of JSON data to 'NSDictionany'
 [result data]
 paymentSeq - generated payment id
 itemSeq - represent item id
 purchaseToken - represent token for validation.
 
 */

+ (instancetype) startPurchaseWithViewController:(UIViewController*)baseViewController
                                          itemId:(long)itemId
                               completionHandler:(TIARequestHandler)handler;

/*!
 @method
 
 @abstract
 Request purchases, Purchases means the payment is successfully completed,
 Does not appear in later history is consumed by 3rd party server.
 
 @param handler     handler's result has that translation of JSON data to 'NSArray'
 [result data of array]
 paymentSeq - generated payment id
 itemSeq - represent item id
 purchaseToken - represent token for validation.
 */

+ (instancetype) purchasesWithCompletionHandler:(TIARequestHandler)handler;

/*!
 @method
 
 @abstract
 Request for item list which is registered in webConsole and in itunesConnect
 
 @param handler     handler's result has that translation of JSON data to 'NSArray'
 [result data of array]
 itemSeq - item id
 itemName - item name
 usingStatus - item status on IAP server
 regYmdt - item registration date on IAP server
 appName - app name
 marketId - market id (AS : APPLE STORE)
 marketItemId - market item id (product id)
 currency - represent to item currency
 price - represent to item price
 */

+ (instancetype) itemListWithCompletionHandler:(TIARequestHandler)handler;


/*!
 // 아직 미정!!
 @method
 
 @abstract
 Request for item list which is registered in webConsole and in itunesConnect
 especially if storeID is 'ONGATE' in testmode, you can get item's currency, price from test server
 
 @param handler     handler's result has that translation of JSON data to 'NSArray'
 [result data of array]
 itemSeq - item id
 itemName - item name
 usingStatus - item status on IAP server
 regYmdt - item registration date on IAP server
 appName - app name
 marketId - market id (AS : APPLE STORE, ONGATE : ONGATE STORE)
 marketItemId - market item id (product id)
 currency - represent to item currency
 price - represent to item price
 */

+ (instancetype) itemListFromTestMode:(BOOL)testMode
                WithCompletionHandler:(TIARequestHandler)handler;




/*!
 @method
 
 @abstract
 Request for item list which is registered in webConsole but not in itunesConnect
 
 @param handler                     handler's result has that translation of JSON data to 'NSArray'
 [result data of array]
 itemSeq - item id
 itemName - item name
 usingStatus - item status on IAP server
 regYmdt - item registration date on IAP server
 appName - app name
 marketId - market id (AS : APPLE STORE)
 marketItemId - market item id (product id)
 */
+ (instancetype) getTCConsoleItemsWithCompletionHandler:(TIARequestHandler)handler;


/*!
 @method
 
 @abstract
 Request processes IncompletePurchases
 
 @param handler             handler's result has that translation of JSON data to 'NSDictionany'
 [result data of NSDictionany]
 successList - success data of 'NSArray'
 failList - fail data of 'NSArray'
 [list data of NSArray(successList or failList)]
 paymentSeq - generated payment id
 itemSeq - represent item id
 purchaseToken - represent token for validation
 marketItemId - market item id (product id)
 currency - represent to item currency
 price - represent to item price
 */

+ (instancetype) processesIncompletePurchasesWithCompletionHandler:(TIARequestHandler)handler;

@end
